!
! Routine to compute the collisional recobination and ionization rates, and
! the collisional cooling rate for ions with super levels.
!
! ME: Added BF_CU, BF_CD, BB_CU and BB_CD parameters, corresponding to the upward
! ME: and downward collisional bound-bound and bound-free transition rates
	SUBROUTINE COLCOOL_SL_V5_ME(CPR,CRR,COOL,CNM,DCNM,
	1             HN_S,HNST_S,dlnHNST_S_dlnT,N_S,
	1             HN_F,HNST_F_ON_S,A_F,W_F,EDGE_F,G_F,LEVNAME_F,
	1             F_TO_S_MAP,N_F,
	1             ZION,ID,COL_FILE,OMEGA_COL,ED,T,ND,
	1             BF_CU,BF_CD,BB_CU,BB_CD)
	IMPLICIT NONE
!       
! Altered 05-Apr-2011 : Updated from V4 
!                          Most changes 02-Dec-10.
!                          HNST replaced in call by HNST_F_ON_S.
!	                   Call to SUBCOL_MULTI_V5 updated (from V4)
! Altered 24-Dec-1996 : SUB_PHOT replaces PHOT_FUN (superficial).
! Altered 24-May-1995 : N_F_MAX removed using F90
! Altered 01-Feb-1996 : Routine now calls SUBCOL_MULTI_V3 (not _V2). The _V3
!                          routine was introdued when interpolationg between
!                          levels in a given super level.
!                          Changed to _V3 for consistency with SUBCOL.
! Created 07-JUn-1995 :Based on COLGENCOOL
!
	EXTERNAL OMEGA_COL
	EXTERNAL ERROR_LU
	INTEGER ERROR_LU	
!
	INTEGER ID
	INTEGER N_S,N_F,ND
	REAL*8 COOL(ND),CRR(ND),CPR(ND)
        ! ME: Added declaration of BF_CU and BF_CD
	REAL*8 BF_CU(N_S,ND),BF_CD(N_S,ND)
	! ME: Added declaration of BB_CU and BB_CD
	REAL*8 BB_CU(N_S,N_S,ND),BB_CD(N_S,N_S,ND)
	REAL*8 CNM(N_S,N_S),DCNM(N_S,N_S)
	REAL*8 ZION,T(ND),ED(ND)
!
	REAL*8 HN_S(N_S,ND),HNST_S(N_S,ND)
	REAL*8 dlnHNST_S_dlnT(N_S,ND)
!
	REAL*8 HN_F(N_F,ND)
	REAL*8 HNST_F_ON_S(N_F,ND)	
	REAL*8 W_F(N_F,ND)
	REAL*8 A_F(N_F,N_F),EDGE_F(N_F),G_F(N_F)
	INTEGER F_TO_S_MAP(N_F)
	CHARACTER*(*) COL_FILE,LEVNAME_F(N_F)
!
	COMMON/CONSTANTS/ CHIBF,CHIFF,HDKT,TWOHCSQ
	REAL*8 CHIBF,CHIFF,HDKT,TWOHCSQ
!
! Local variables.
!
	REAL*8 H,TMP_ED
	! ME: Added K
	INTEGER I,J,K,IONE
	PARAMETER (IONE=1)
!
	REAL*8 OMEGA_F(N_F,N_F)
	REAL*8 dln_OMEGA_F_dlnT(N_F,N_F)
!
	H=6.6261965D-12		!H*1.0E+15  (1.0E+15 due to times frequency)
	TMP_ED=1.0D0
!
	DO I=1,ND			!Which depth
!                            
! Compute collisional cross-sections (and their T derivatives)
!
	  COOL(I)=0.0D0
	  CALL SUBCOL_MULTI_V5(OMEGA_F,dln_OMEGA_F_dlNT,
	1          CNM,DCNM,
	1          HN_S(1,I),HNST_S(1,I),dlnHNST_S_dlnT(1,I),N_S,
	1          HN_F(1,I),HNST_F_ON_S(1,I),W_F(1,I),EDGE_F,
	1          A_F,G_F,LEVNAME_F,N_F,
	1          ZION,ID,COL_FILE,OMEGA_COL,
	1          F_TO_S_MAP,COOL(I),T(I),TMP_ED,IONE)
!                        
	  CPR(I)=0.0D0
	  CRR(I)=0.0D0
	  COOL(I)=COOL(I)*ED(I)*H
!
	  DO J=1,N_S				!Level
	    CPR(I)=CPR(I)+HN_S(J,I)*(ED(I)*CNM(J,J))
	    CRR(I)=CRR(I)+HNST_S(J,I)*(ED(I)*CNM(J,J))
	  END DO

          ! ME: Added calculation of BF_CU, BF_CD, BB_CU and BB_CD
	  DO J=1,N_S
	    BF_CU(J,I)=HN_S(J,I)*(ED(I)*CNM(J,J))
	    BF_CD(J,I)=HNST_S(J,I)*(ED(I)*CNM(J,J))
	    DO K=1,N_S
	       BB_CU(J,K,I)=HN_S(J,I)*(ED(I)*CNM(J,K))
	       BB_CD(J,K,I)=HN_S(K,I)*(ED(I)*CNM(K,J))
	    END DO
	  END DO	 
!
	END DO
!
	RETURN
	END
